#!/bin/bash
#####################################################################
####### G15 Amarok Plugin Plus v1.1 (released under GPL 2.0) ########
#####################################################################
# Author:                        Robert Riemann                     #
# Report bugs and requests to:   robert.riemann@physik.hu-berlin.de #
# Requires:                      g15composer, amarok with dcop      #
#                                kmix with dcop (switch -g, option) #
#####################################################################
# The script shows the meta info to the actual track of amarok      #
# meta infos on screen: title, artist, album, current-/total-time,  #
#                       rating, bitrate and filetype                #
#####################################################################

# Removes all background progresses on exit
function cleanup() {
	if [ -n "$composer_pid" ]; then
		kill $composer_pid && unset composer_pid; fi
	if [ -n "$refresh_pid" ]; then
		kill $refresh_pid && unset refresh_pid; fi
	if [ -p "$pipe" ]; then
		rm $pipe; fi
	if [ -f "$tmpfile" ]; then
		rm $tmpfile; fi
	exit
}

# Get control about the signals to stop the g15composer and remove the pipe
trap cleanup EXIT HUP INT QUIT FPE KILL TERM


# Check installation of g15composer
if [ -z $(which g15composer) ]; then
	echo 'g15composer could not be found. Aborting.'
	exit 1
fi

# Read Options
while getopts ":p:t:s:cg" opt; do
	case $opt in
		p )	if [ -p $opt ]; then
				pipe=$OPTARG
			else
				echo "$opt wasn't found or is no pipe. Aborting."
				exit 1
			fi;;
		t )	if [ ! -e $opt ]; then
				tmpfile=$OPTARG
			else
				echo "$opt already exists. Aborting."
				exit 1
			fi;;
		s )	if [ -n $OPTARG ]; then
				speed=$OPTARG
			else
				echo "Speed has to be a positive number (real). Aborting."
				exit 1
			fi;;
		c )	consolemod="0" ;;
		g )	globalvol="0" ;;
		* )	echo -e 'Usage:\tamarok_script_plus.sh [-h] [-p Pipe] [-s speed]'
			echo -e '\t-h : Print this help'
			echo -e '\t-p : Sets pipe to use (default: /tmp/g15amarokpluginplus_pipe)'
			echo -e '\t-t : Sets temporary file to use (default: /tmp/g15amarokpluginplus_do, must not exist)'
			echo -e '\t-c : Go in console mode: no need to run as Amarok plugin for getting volume Change screen'
			echo -e '\t-g : Shows change of the kmix main channel volume. Only in console mode possible.'
			echo -e '\t-s : Set speed to refresh the screen (in seconds, values like 0.5 are possible)'
			exit 1;;
	esac
	if [ $globalvol ] &&  [ ! $consolemod ]; then
		echo -e "Global volume mode only in console mode. Aborting.\n(Usage: $0 -h)"
		exit 1
	fi
done

# Set standard values for pipe ( /tmp/g15amarokpluginplus_pipe ), tmporary file ( /tmp/g15amarokpluginplus_do ) and speed ( 1 second )
pipe=${pipe:-'/tmp/g15amarokpluginplus_pipe'}
tmpfile=${tmpfile:-'/tmp/g15amarokpluginplus_do'}
true > $tmpfile
speed=${speed:-'1'}

# Check for an already running g15composer with our pipe - Normally there is'nt one
composer_pid=$(pgrep -f -x "g15composer $pipe")
if [ -z $composer_pid ]; then
	g15composer $pipe &
	composer_pid=$!
fi
echo "g15composer - PID: $composer_pid"

# enables different modes (show the volume screen, or the title information, or ...)
function getevents() {
	# Sets screen mode to begin (show title informations)
	amarok_stdin_last="showInfos"

	# Counts the age of the last screen mode change in seconds
	age="0"

	# Sets the volume Screen duration for the different modes (console or amarok)
	showtime="3"
	showtime=${consolemod+"8"}

	# Sets start volume for the console mode
	if [ $consolemod ]; then
		if [ ! $globalvol ]; then
			vol_last=$(dcop amarok player getVolume)
		else
			vol_last=$(dcop kmix Mixer0 masterVolume)
		fi
	fi

	while true; do		
		 # If not in consolemode, so listen to amarok for volume change notification
		if [ ! $consolemod ]; then
			if read -t 1 amarok_stdin; then
				amarok_stdin_last=$amarok_stdin
				age="0"
			else
				age=$(( $age + 1 ))
			fi
		# if in console mode, so catch the volume changes from dcop, if globalvol is on, than from kmix
		else
			if [ ! $globalvol ]; then
				vol=$(dcop amarok player getVolume)
			else
				vol=$(dcop kmix Mixer0 masterVolume)
			fi
			if [ $(( $vol )) -ne $(( $vol_last )) ]; then
				amarok_stdin_last="volumeChange: $vol"
				age="0"
			else
				age=$(( $age + 1 ))
			fi
			vol_last=$vol
			sleep 0.25
		fi

		# Figure out, which screen is the next
		if [ "${amarok_stdin_last%:*}" = "volumeChange" ] && [ $(( $age )) -gt $(( $showtime )) ]; then
			if [ $(dcop amarok player status) != 0 ]; then
				amarok_stdin_last="showInfos"
			else
				amarok_stdin_last="notPlaying"
			fi
		elif [ ! "${amarok_stdin_last%:*}" = "volumeChange" ]; then
			if [ $(dcop amarok player status) != 0 ]; then
				amarok_stdin_last="showInfos"
			else
				amarok_stdin_last="notPlaying"
			fi
		fi

		# Write the mode in a temporary file, so that the background process can read it
		echo "$amarok_stdin_last" > "$tmpfile"
	done
}

function screenrefresh() {
	# startposition for scrolling the title/band/album
	post="0"
	posb="0"
	posa="0"

	while true; do
		# Read the next mode from the temporary file
		amarok_stdin_last=$(cat "$tmpfile")
		# Shows the volume change message
		if [ "${amarok_stdin_last%:*}" = "volumeChange" ]; then

			# Reads the following lines up to EOF and sends it to our pipe
			cat > $pipe << EOF
MC 1
PB 0 0 159 42
TO 1 2 2 1 "volume Change"
DB 10 20 149 30 1 ${amarok_stdin_last#* } 100 2
TO 1 32 2 1 "${amarok_stdin_last#* }%"
MC 0
EOF
	
		# Show the meta infos if amarok is'nt stopped
		elif [ "$amarok_stdin_last" = "showInfos" ]; then
			# Prepares the scrolling of title, artist/band and album
			title=$(dcop amarok player title)
			if [ ${#title} -gt 16 ] && [ $((${#title} - $post)) -gt 0 ] && [ "$title" = "$titlelast" ]; then
				post=$((post + 1)); else post="0"; fi; titlelast=$title
			artist=$(dcop amarok player artist)
			if [ ${#artist} -gt 16 ] && [ $((${#artist} - $posb)) -gt 0 ] && [ "$artist" = "$artistlast" ]; then
				posb=$((posb + 1)); else posb="0"; fi; artistlast=$artist
			album=$(dcop amarok player album)
			if [ ${#album} -gt 16 ] && [ $((${#album} - $posa)) -gt 0 ] && [ "$album" = "$albumlast" ]; then
				posa=$((posa + 1)); else posa="0"; fi; albumlast=$album
	
			rating=$(dcop amarok player rating)
			bitrate=$(dcop amarok player bitrate)

			# Reads the following lines up to EOF and sends it to our pipe
			cat > $pipe << EOF
MC 1
TO 2 2 1 0 "Track "
TO 31 2 2 0 "${title:$post:16}"
DB 0 11 159 11 1 $(dcop amarok player trackCurrentTime) $(dcop amarok player trackTotalTime) 1
TO 2 15 1 0 "Band  "
TO 31 15 2 0 "${artist:$posb:16}"
TO 2 24 1 0 "Album "
TO 31 24 2 0 "${album:$posa:16}"
DL 1 32 158 32 1
TO 2 34 1 0 "$(dcop amarok player currentTime)/$(dcop amarok player totalTime)"
TO 2 34 1 1 "rating:$(printf "%2d" $rating)"
TO 2 34 1 2 "$(dcop amarok player type)${bitrate:+"/($bitrate)"}"
PB 0 0 159 42
MC 0
EOF
	
		# Shows the amarok stopped status screen
		elif [ "$amarok_stdin_last" = "notPlaying" ]; then
			# Reads the following lines up to EOF and sends it to our pipe
			cat > $pipe << EOF
MC 1
PB 0 0 159 42
TO 1 2 2 1 "not playing"
MC 0
EOF
		fi
		sleep $speed

	done
}

# Start the background process which refreshes the display and save its process id for killing it on exit
screenrefresh &
refresh_pid=$!

# Start the process (function), which controls the screen mode
getevents

exit 0